CREATE TABLE DEPT (
    DEPTNO  INTEGER NOT NULL PRIMARY KEY,
    DNAME   VARCHAR(14),
    LOC     VARCHAR(13)
);

COMMIT;

CREATE TABLE EMP (
    EMPNO     INTEGER NOT NULL PRIMARY KEY,
    ENAME     VARCHAR(10),
    JOB       VARCHAR(9),
    MGR       INTEGER,
    HIREDATE  TIMESTAMP,
    SAL       INTEGER,
    COMM      INTEGER,
    DEPTNO    INTEGER REFERENCES DEPT (DEPTNO)
);

CREATE TABLE IBDAC_BLOB (
  ID integer NOT NULL PRIMARY KEY,
  Title VARCHAR(30),
  Pic BLOB
);

CREATE TABLE CRGRID_TEST (
  Id NUMERIC(4) NOT NULL PRIMARY KEY,
  Name VARCHAR(10),
  Country VARCHAR(30),
  City VARCHAR(30),
  Street VARCHAR(30),
  BirthDate TIMESTAMP,
  Job VARCHAR(9),
  Hiredate TIMESTAMP,
  Sal NUMERIC(7, 2),
  Remarks BLOB SUB_TYPE TEXT);

CREATE TABLE IBDAC_LONG_CHAR (
  Code INTEGER NOT NULL PRIMARY KEY,
  Title VARCHAR(30),
  Val VARCHAR(4000)
);

CREATE TABLE IBDAC_PICTURES (
  Name VARCHAR(50),
  Picture BLOB
);

CREATE TABLE IBDAC_BLOB_TEXT (
  Code INTEGER NOT NULL PRIMARY KEY,
  Title VARCHAR(30),
  Val BLOB SUB_TYPE text
);

INSERT INTO DEPT VALUES
  (10,'ACCOUNTING','NEW YORK');
INSERT INTO DEPT VALUES
  (20,'RESEARCH','DALLAS');
INSERT INTO DEPT VALUES
  (30,'SALES','CHICAGO');
INSERT INTO DEPT VALUES
  (40,'OPERATIONS','BOSTON');
INSERT INTO EMP VALUES
  (7369,'SMITH','CLERK',7902,'17.12.1980',800,NULL,20);
INSERT INTO EMP VALUES
  (7499,'ALLEN','SALESMAN',7698,'20.02.1981',1600,300,30);
INSERT INTO EMP VALUES
  (7521,'WARD','SALESMAN',7698,'22.02.1981',1250,500,30);
INSERT INTO EMP VALUES
  (7566,'JONES','MANAGER',7839,'02.04.1981',2975,NULL,20);
INSERT INTO EMP VALUES
  (7654,'MARTIN','SALESMAN',7698,'28.09.1981',1250,1400,30);
INSERT INTO EMP VALUES
  (7698,'BLAKE','MANAGER',7839,'01.05.1981',2850,NULL,30);
INSERT INTO EMP VALUES
  (7782,'CLARK','MANAGER',7839,'09.06.1981',2450,NULL,10);
INSERT INTO EMP VALUES
  (7788,'SCOTT','ANALYST',7566,'13.07.87',3000,NULL,20);
INSERT INTO EMP VALUES
  (7839,'KING','PRESIDENT',NULL,'17.11.1981',5000,NULL,10);
INSERT INTO EMP VALUES
  (7844,'TURNER','SALESMAN',7698,'08.09.1981',1500,0,30);
INSERT INTO EMP VALUES
  (7876,'ADAMS','CLERK',7788,'13.07.87',1100,NULL,20);
INSERT INTO EMP VALUES
  (7900,'JAMES','CLERK',7698,'03.12.1981',950,NULL,30);
INSERT INTO EMP VALUES
  (7902,'FORD','ANALYST',7566,'03.12.1981',3000,NULL,20);
INSERT INTO EMP VALUES
  (7934,'MILLER','CLERK',7782,'23.01.1982',1300,NULL,10);

INSERT INTO CRGRID_TEST (Id, Name, Country, City, Street, BirthDate, Job, HireDate, Sal) VALUES
  (5001, 'SMITH', 'ENGLAND', 'LONDON', 'BOND st.', '12.10.63', 'CLERK', '17.12.80', 800);

INSERT INTO CRGRID_TEST (Id, Name, Country, City, Street, BirthDate, Job, HireDate, Sal) VALUES
  (5002, 'ALLEN', 'ENGLAND', 'LONDON', 'BAKER st.', '04.03.61', 'SALESMAN', '20.02.81', 1600);

INSERT INTO CRGRID_TEST (Id, Name, Country, City, Street, BirthDate, Job, HireDate, Sal) VALUES
  (5003, 'MARTIN', 'FRANCE', 'LION', 'WEAVER st.', '23.01.57', 'MANAGER', '02.04.81', 2900);

INSERT INTO IBDAC_LONG_CHAR VALUES
  (5001, 'Record1', 'LongChar1');
INSERT INTO IBDAC_LONG_CHAR VALUES
  (5002, 'Record2', 'LongChar2');
INSERT INTO IBDAC_LONG_CHAR VALUES
  (5003, 'Record3', 'LongChar3');

CREATE TABLE IBDAC_ARRAYS (
    ID            INTEGER NOT NULL,
    INT_ARR1      INTEGER [1:5],
    CHAR_ARR2     CHAR(10)[1:4, 1:3],
    INT_ARR3      INTEGER [1:4, 1:2, 1:3]
);

ALTER TABLE IBDAC_ARRAYS
ADD CONSTRAINT IDDAC_ARRAYS_PK PRIMARY KEY (ID);

CREATE GENERATOR IBDAC_GEN;

CREATE TABLE DB_KEY_TABLE (
  TITLE           VARCHAR(16),
  VAL             VARCHAR(32),
  STAMP_ADD       TIMESTAMP
);

CREATE TABLE THREAD_TABLE (
  ID    INTEGER,
  NAME  VARCHAR(30)
);

CREATE TABLE IBDAC_Loaded (
  Code INTEGER,
  Dbl  DOUBLE PRECISION,
  Str  VARCHAR(50),
  Dat  DATE
);

SET TERM ^;

CREATE PROCEDURE TEST_PARAMS 
RETURNS (
    PO VARCHAR(30))
AS
begin
  PO = 'This is text parameter';
end^

CREATE PROCEDURE TEST_SUSPEND (
    MAXID INTEGER)
RETURNS (
    ID INTEGER)
AS
begin
  ID = 0;
  while (ID < MaxID) do
  begin
    ID = ID + 1;
    suspend;
  end
end^

CREATE PROCEDURE IBDAC_BLOB_INSERT(
    p_ID INTEGER,
    p_Title VARCHAR(200),
    p_Pic BLOB
)
as
begin
  INSERT INTO IBDAC_BLOB(ID, Title, Pic)
  VALUES (:p_ID, :p_Title, :p_Pic);
end^

CREATE PROCEDURE COUNT_BY_SAL(
    SAL INTEGER)
RETURNS (
    CNT INTEGER)
AS
begin
    select count(*) as EmpCount from emp where sal > :sal into :cnt;
end^

CREATE PROCEDURE SEL_FROM_EMP 
RETURNS (
    EMPNO INTEGER,
    ENAME VARCHAR(10),
    JOB VARCHAR(9),
    MGR INTEGER,
    SAL INTEGER,
    COMM INTEGER,
    DEPTNO INTEGER)
AS
BEGIN
  FOR  SELECT EMPNO, ENAME, JOB, MGR, SAL, COMM, DEPTNO FROM emp
    INTO :EMPNO, :ENAME, :JOB, :MGR, :SAL, :COMM, :DEPTNO
  DO
    suspend;
END^

CREATE PROCEDURE IBDAC_SEL_FROM_TEXT_BLOB (
    CODE INTEGER)
RETURNS (
    TITLE VARCHAR(30),
    VAL BLOB SUB_TYPE 0 SEGMENT SIZE 80)
AS
begin
  select Title, val
    from ibdac_blob_text
    where code = :code
    into :title, :val;
  suspend;
end
^

SET TERM ;^

COMMIT;